import unittest

from pyradios.radios import EndPointBuilder


class TestEndPoint(unittest.TestCase):
    def setUp(self):
        self.builder = EndPointBuilder(fmt="json")

    def test_countries(self):
        expected = "json/countries"
        result = self.builder.produce_endpoint(endpoint="countries")
        self.assertEqual(result, expected)

    def test_countries_with_filter(self):
        expected = "json/countries/Brazil"
        result = self.builder.produce_endpoint(
            endpoint="countries", filter="Brazil"
        )
        self.assertEqual(result, expected)

    def test_codecs(self):
        expected = "json/codecs"
        result = self.builder.produce_endpoint(endpoint="codecs")
        self.assertEqual(result, expected)

    def test_codecs_with_filter(self):
        expected = "json/codecs/mp3"
        result = self.builder.produce_endpoint(endpoint="codecs", filter="mp3")
        self.assertEqual(result, expected)

    def test_states(self):
        expected = "json/states"
        result = self.builder.produce_endpoint(endpoint="states")
        self.assertEqual(result, expected)

    def test_states_with_filters(self):
        # falha quando params ñ são nomeados
        expected = "json/states/German/ber"
        result = self.builder.produce_endpoint(
            endpoint="states", filter="ber", country="German"
        )
        self.assertEqual(result, expected)

    def test_languages(self):
        expected = "json/languages"
        result = self.builder.produce_endpoint(endpoint="languages")
        self.assertEqual(result, expected)

    def test_languages_with_param(self):
        expected = "json/languages/ger"
        result = self.builder.produce_endpoint(
            endpoint="languages", filter="ger"
        )
        self.assertEqual(result, expected)

    def test_tags(self):
        expected = "json/tags"
        result = self.builder.produce_endpoint(endpoint="tags")
        self.assertEqual(result, expected)

    #
    def test_tags_with_param(self):
        expected = "json/tags/jazz"
        result = self.builder.produce_endpoint(endpoint="tags", filter="jazz")
        self.assertEqual(result, expected)

    #
    def test_stations(self):
        expected = "json/stations"
        result = self.builder.produce_endpoint(endpoint="stations")
        self.assertEqual(result, expected)

    def test_stations_byid(self):
        expected = "json/stations/byid/0000"
        result = self.builder.produce_endpoint(
            endpoint="stations", by="byid", search_term="0000"
        )
        self.assertEqual(result, expected)

    def test_stations_byuuid(self):
        expected = "json/stations/byuuid/0000"
        result = self.builder.produce_endpoint(
            endpoint="stations", by="byuuid", search_term="0000"
        )
        self.assertEqual(result, expected)

    def test_stations_byname(self):
        expected = "json/stations/byname/jazz"
        result = self.builder.produce_endpoint(
            endpoint="stations", by="byname", search_term="jazz"
        )
        self.assertEqual(result, expected)

    def test_playable_station(self):
        expected = "v2/json/url/123"
        result = self.builder.produce_endpoint(
            endpoint="playable_station", station_id=123, ver="v2"
        )
        self.assertEqual(result, expected)

    def test_station_search(self):
        expected = "json/stations/search"
        result = self.builder.produce_endpoint(endpoint="station_search")
        self.assertEqual(result, expected)



from urllib.parse import urljoin
from xml.etree import ElementTree

import requests

from pyradios.base_url import pick_url
from pyradios.constants import endpoints


def request(endpoint, **kwargs):

    BASE_URL = pick_url(
        filename="data.cache", expire=kwargs.get("expire", 604800), **kwargs
    )

    fmt = kwargs.get("format", "json")

    if fmt == "xml":
        content_type = f"application/{fmt}"
    else:
        content_type = f"application/{fmt}"

    headers = {"content-type": content_type, "User-Agent": "pyradios/dev"}

    params = kwargs.get("params", {})

    url = BASE_URL + endpoint

    resp = requests.get(url, headers=headers, params=params)

    if resp.status_code == 200:
        if fmt == "xml":
            return resp.text
        return resp.json()

    return resp.raise_for_status()


class EndPointBuilder:
    def __init__(self, **kwargs):
        self.fmt = kwargs.get("fmt")
        self._option = None
        self._endpoint = None

    @property
    def endpoint(self):
        return endpoints[self._endpoint][self._option]

    def produce_endpoint(self, **parts):
        self._option = len(parts)
        self._endpoint = parts["endpoint"]
        parts.update({"fmt": self.fmt})
        return self.endpoint.format(**parts)


class RadioBrowser:
    """
    Supported output formats: JSON

    TODO:
        * XML support.
    """

    config = {}

    def __init__(self, fmt="json", **kwargs):
        self.config.update(kwargs, fmt=fmt)
        self.builder = EndPointBuilder(**self.config)

    def countries(self, filter=""):
        endpoint = self.builder.produce_endpoint(endpoint="countries")
        return request(endpoint, **self.config)

    def codecs(self, filter=""):
        endpoint = self.builder.produce_endpoint(endpoint="codecs")
        return request(endpoint, **self.config)

    def states(self, country="", filter=""):
        endpoint = self.builder.produce_endpoint(
            endpoint="states", country=country, filter=filter
        )
        return request(endpoint, **self.config)

    def languages(self, filter=""):
        endpoint = self.builder.produce_endpoint(
            endpoint="languages", filter=filter
        )
        return request(endpoint, **self.config)

    def tags(self, filter=""):
        endpoint = self.builder.produce_endpoint(
            endpoint="tags", filter=filter
        )
        return request(endpoint, **self.config)

    def stations_byid(self, id):
        endpoint = self.builder.produce_endpoint(
            endpoint="stations", by="byid", search_term=id
        )
        return request(endpoint, **self.config)

    def stations_byuuid(self, uuid):
        endpoint = self.builder.produce_endpoint(
            endpoint="stations", by="byuuid", search_term=uuid
        )
        return request(endpoint, **self.config)

    def stations_byname(self, name):
        endpoint = self.builder.produce_endpoint(
            endpoint="stations", by="byname", search_term=name
        )
        return request(endpoint, **self.config)

    def stations_bynameexact(self, nameexact):
        endpoint = self.builder.produce_endpoint(
            endpoint="stations", by="bynameexact", search_term=nameexact
        )
        return request(endpoint, **self.config)

    def stations_bycodec(self, codec):
        endpoint = self.builder.produce_endpoint(
            endpoint="stations", by="bycodec", search_term=codec
        )
        return request(endpoint, **self.config)

    def stations_bycodecexact(self, codecexact):
        endpoint = self.builder.produce_endpoint(
            endpoint="stations", by="bycodecexact", search_term=codecexact
        )
        return request(endpoint, **self.config)

    def stations_bycountry(self, country):
        endpoint = self.builder.produce_endpoint(
            endpoint="stations", by="bycountry", search_term=country
        )
        return request(endpoint, **self.config)

    def stations_bycountryexact(self, countryexact):
        endpoint = self.builder.produce_endpoint(
            endpoint="stations", by="bycountryexact", search_term=countryexact
        )
        return request(endpoint, **self.config)

    def stations_bystate(self, state):
        endpoint = self.builder.produce_endpoint(
            endpoint="stations", by="bystate", search_term=state
        )
        return request(endpoint, **self.config)

    def stations_bystateexact(self, stateexact):
        endpoint = self.builder.produce_endpoint(
            endpoint="stations", by="bystateexact", search_term=stateexact
        )
        return request(endpoint, **self.config)

    def stations_bylanguage(self, language):
        endpoint = self.builder.produce_endpoint(
            endpoint="stations", by="bylanguage", search_term=language
        )
        return request(endpoint, **self.config)

    def stations_bylanguageexact(self, languageexact):
        endpoint = self.builder.produce_endpoint(
            endpoint="stations",
            by="bylanguageexact",
            search_term=languageexact,
        )
        return request(endpoint, **self.config)

    def stations_bytag(self, tag):
        endpoint = self.builder.produce_endpoint(
            endpoint="stations", by="bytag", search_term=tag
        )
        return request(endpoint, **self.config)

    def stations_bytagexact(self, tagexact):
        endpoint = self.builder.produce_endpoint(
            endpoint="stations", by="bytagexact", search_term=tagexact
        )
        return request(endpoint, **self.config)

    def playable_station(self, station_id):
        endpoint = self.builder.produce_endpoint(
            endpoint="playable_station", station_id=station_id, ver="v2"
        )

        return request(endpoint, **self.config)

    def stations(self, **params):
        endpoint = self.builder.produce_endpoint(endpoint="stations")
        if params:
            self.config.update({"params": params})
        return request(endpoint, **self.config)

    def station_search(self, params={}, **kwargs):
        #  http://www.radio-browser.info/webservice#Advanced_station_search
        assert isinstance(params, dict), "params must be a dictionary."
        endpoint = self.builder.produce_endpoint(endpoint="station_search")
        if params:
            self.config["params"] = params
        if kwargs:
            self.config["params"].update(kwargs)
        return request(endpoint, **self.config)



"""
Get all base urls of all currently available radiobrowser servers

"""
import random
import socket

from pyradios.cache import cache
from pyradios.log import logger


def fetch_all_hosts():
    """
    Get all ip of all currently available radiobrowser servers.

    Returns:
        list: a list of strings

    """
    try:
        data = socket.getaddrinfo(
            "all.api.radio-browser.info", 80, 0, 0, socket.IPPROTO_TCP
        )
    except socket.gaierror as exc:
        logger.exception("Networking failure:")
    ips = [ip[4][0] for ip in data]
    return ips


def rdns_lookup(ip):
    """
    Do a reverse dns lookup.

    Resolves domain names into associated IP addresses.

    Returns:
        str: domain name

    """
    try:
        name = socket.gethostbyaddr(ip)
    except socket.herror as exc:
        logger.exception("Networking failure:")
    return name[0]


def pick_url(filename, expire, **kwargs):
    @cache(filename=filename, expire=expire, **kwargs)
    def fetch_hosts():
        return list({rdns_lookup(ip) for ip in fetch_all_hosts()})

    names = fetch_hosts()
    names.sort()
    url = random.choice(names)
    return "https://{}/".format(url)



from pathlib import Path

from pyradios.config import app_dirs


def create_app_dirs(filename, path):
    p = Path(path)
    if not p.exists():
        p.mkdir(parents=True)
    return p / filename


def setup_log_file(filename, **kwargs):
    return create_app_dirs(filename, app_dirs.user_log_dir)


def setup_cache_file(filename, **kwargs):
    return create_app_dirs(filename, app_dirs.user_cache_dir)
